/*
  -------------------- file indep.c
  Created 2 Aug 94 by Andrew Shapira, Rensselaer Polytechnic Institute.
  Functions related to making results of generators independent across
  two runs when the random number generator starts in two different states.

	Copyright (C) 1994 Andrew Shapira.

	This file is part of the Ntuplib library.

	Ntuplib is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	Ntuplib is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with the Ntuplib Library; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "our_common.h"
#include "indep.h"

/*  Take the array [0 1 ... k-1] and proceed left to right, swapping
    element i with element i+1 with 50% probability.  Then make a pass
    from right to left.  Thus any given element will have a nonzero
    probability of ending up in a particular slot.
*/
void
indep_perm(
int * const perm,
const int k,
long (* const randfunc) P(( void ))
)
{
  int i;

  for (i = 0; i < k; i++)
    perm[i] = i;
  
  for (i = 0; i < k-1; i++) {
    if ((*randfunc) () & 1) {
      const int swap = perm[i];
      perm[i] = perm[i+1];
      perm[i+1] = swap;
    }
  }

  for (i = k-2; i >= 0; i--) {
    if ((*randfunc) () & 1) {
      const int swap = perm[i];
      perm[i] = perm[i+1];
      perm[i+1] = swap;
    }
  }
}
