/*
  -------------------- file backgen.c
  Created 2 Jun 94 by Andrew Shapira, Rensselaer Polytechnic Institute.

	Copyright (C) 1994 Andrew Shapira.

	This file is part of the Ntuplib library.

	Ntuplib is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	Ntuplib is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with the Ntuplib Library; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "our_common.h"
#include "backgen.h"

/*  Generate the next partial solution, given the previous partial
    solution.

    If we found another partial solution to look at, then:
        Sets *success_ret to 1.
        Sets *k_ret to the new search level
    If no more partial solutions to look at, then:
        Sets *success_ret to 0.
        Value of *k_ret is undefined.
*/
void
backgen_next(
const int max_id,
const int max_color_index,  /* 0 or 1 */
const int n,                /* n-tuple at most */
const int oldpassed,        /* did previous tuple pass constraints? */
/* we read and set k_ret, ids, color_indices */
int * const k_ret,          /* search level */
int * const ids,            /* ids of n-tuple */
int * const color_indices,  /* color indices of n-tuple */
int * const success_ret
/*  Before the first call, the caller should set:
  *k_ret = 0
  *pi_ids = 0
  *color_indices[0] = 0 (if color_indices is not NULL)
*/
)
{
  int k;

  k = *k_ret;
  *success_ret = 1;

  if (oldpassed && (k < n-1)) {
    k++;
    /* set things up for first time through loop below */
    color_indices[k] = max_color_index;
    ids[k] = ids[k-1];
  }

  do {
    if (color_indices[k] < max_color_index)
      {  color_indices[k]++;  goto done;  }
    if (ids[k] < max_id) {
      ids[k]++;
      if (color_indices)  color_indices[k] = 0;
      goto done;
    }
    k--;
  } while (k >= 0);
  *success_ret = 0;

done:
  *k_ret = k;
  return;
}
