/*
  -------------------- file pi.h
  Created 17 May 94 by Andrew Shapira, Rensselaer Polytechnic Institute.

	Copyright (C) 1994 Andrew Shapira.

	This file is part of the Ntuplib library.

	Ntuplib is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	Ntuplib is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with the Ntuplib Library; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef _170594_221531
#define _170594_221531

#include "../libinclude/ntup.h"
#include "coord.h"
#define COORD_DEBUG

#ifdef COORD_DEBUG
void debug_coord P(( const struct coord_struct * const));
void debug_piidc_vec P(( const struct coord_struct * const,
  const char * const, const int * const, const int ));
#endif  /*COORD_DEBUG*/

/* A method for storing (piid,color) pairs.
   A "piidc" is an integer that represents a pi id and a color.
   Note: piidcs are "int"s, even though at present they are never negative.
   In some places it is important that they be nonnegative; in such
   a place there should be an ASSERTION that the piidc used is nonnegative.
   They are "int"s instead of "unsigned"s to allow for the future
   possibility of using negative piidc's, and for historical reasons.

   The following functions convert back and forth between piidc's and pairs.
   Since COLORS is a power of 2, these functions are fast.
*/
/* make piidc from pi id ii and color cc */
#define PIID_C_TO_PIIDC(ii,cc)    (((ii)*COLORS)+(cc))
/* get pi id from piidc xx */
#define PIIDC_TO_PIID(xx)         (((xx) - ((xx) % COLORS)) / COLORS)
/* get color from piidc xx */
#define PIIDC_TO_COLOR(xx)        ((xx) % COLORS)

void pi_make_standard P(( const int, const int, struct coord_struct * const,
  void  *(*) P((const int)) ));
void pi_make_random P(( struct coord_struct *, const struct coord_struct *,
  void *(*) P((const int)) ));
void piidc_to_pix P(( const struct coord_struct * const,
  const int, struct pix_struct * const ));
void convert_ps_to_pixels P(( const struct coord_struct * const,
    const int * const, const int, struct pix_struct * const ));
void ps_print P(( FILE *, const struct coord_struct *const,
  const int * const, const int, const int ));

#endif /* _170594_221531 */
