/*
  -------------------- file permute.c

	Copyright (C) 1992 Andrew Shapira.

	This file is part of the Ntuplib library.

	Ntuplib is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	Ntuplib is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with the Ntuplib Library; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#include <stdio.h>
#include <malloc.h>
#include "our_common.h"
#include "memalloc.h"
#include "matrix.h"
#include "permute.h"

/*
  Return a random permutation of 0 .. t-1 in x[0], x[1], ..., x[t-1].
  From Knuth Volume 2 1981 p139.

  randfunc is a function that has the same calling sequence
  as random().  It may be random() itself.

  long  random();
*/
void
randperm(
int         * const x,
const int   t,
long        (*randfunc) P(())
)
{
  int   j;

	if (t < 1)  return;

  /* AS: shift so as to reduce position bias from random() */
	/* I don't know if this is really necessary.  It may have originally
		 been introduced because of a bug in the code farther down in
		 this function.  Or, it might still be needed.  COULDDO: find out
		 whether this is needed.  - AS 12Oct95
		 If it is needed then we can get an even better preprocessing step
		 by first setting x[j] = (j*p) mod t, where p is a prime
		 relatively prime to t.  This requires some work to handle
		 overflow, and probably isn't worth doing.
	*/
	{
		const long shift = (*randfunc) () % t;
		for (j = 0; j < t; j++)
			x[(j+shift) % t] = j;
	}

  for (j = t-1; j; j--) {
    const long k = (*randfunc) () % (j+1);
		/* (code below is ok when j=k) */
		const long swap = x[j];
		x[j] = x[k];
		x[k] = swap;
  }
}
